local world = require('world')
local ground = require('entities/ground')

local playerWidth = 30
local playerHeight = 80
local originalPlayerWidth = playerWidth
local originalPlayerHeight = playerHeight
local playerSpeed = 2000
local speedSlide = 5
local playerMaxSpeed = 1200
local playerJump = 500

touchingGround = false
local isSliding = false
local speedDecreaseSlide = 3000

local testValue = 0

local player = {}

function changeCollider(x, y, width, height)
  if player.body or player.fixture then
    player.fixture:destroy()
    player.body:destroy()
  end

  player.body = love.physics.newBody(world, x, y, 'dynamic')
  player.body:setFixedRotation(true)
  player.shape = love.physics.newRectangleShape(width, height)
  player.fixture = love.physics.newFixture(player.body, player.shape)
end

changeCollider(love.graphics.getWidth()/2, love.graphics.getHeight()/2, playerWidth, playerHeight)

function player.draw(self)
  love.graphics.setColor(0, 0.3, 1)
  x, y = self.body:getWorldPoints(self.shape:getPoints())
  love.graphics.rectangle('fill', x, y, playerWidth, playerHeight)
end

function player.update(self, dt)
  contacts = world:getContacts()

  for test, contact in ipairs(contacts) do
    fix1, fix2 = contact:getFixtures()
    if (fix1:getCategory() == 2 or fix2:getCategory() == 2) and contact:isTouching() then
      touchingGround = true
    else
      touchingGround = false
    end
  end

  testValue = testValue + 1

  playerMovement(self, dt, touchingGround)
end

function playerMovement(self, dt, groundCheck)
  --According to forum, dt is not needed

  vx, vy = self.body:getLinearVelocity()

  if vx >= playerMaxSpeed or vx <= -playerMaxSpeed then
    print("Max Speed Reached" .. testValue)
    testValue = testValue + 1
  end

  if love.keyboard.isDown("d") and vx < playerMaxSpeed then
    if isSliding == false then
      if vx < 0 then
        self.body:applyForce(playerSpeed - speedSlide, 0)
      else
        self.body:applyForce(playerSpeed, 0)
      end
    end
    if isSliding == true and vx > 1 then
      self.body:applyForce(playerSpeed - speedDecreaseSlide, 0)
    else
      self.body:applyForce(0, 0)
    end
  end

  if love.keyboard.isDown("a") and vx > -playerMaxSpeed then
    if isSliding == false then
      if vx > 0 then
        self.body:applyForce(-playerSpeed + speedSlide, 0)
      else
        self.body:applyForce(-playerSpeed, 0)
      end
    end
    if isSliding == true and vx < 1 then
      self.body:applyForce(-playerSpeed + speedDecreaseSlide, 0)
    else
      self.body:applyForce(0, 0)
    end
  end

  --Variable jump height?
  if love.keyboard.isDown("w") and groundCheck == true then
    self.body:applyLinearImpulse(0, -playerJump)
  end

------------------------
  if love.keyboard.isDown("s") and groundCheck == true and vx~= 0 then
    if isSliding == false then
      isSliding = true
      playerHeight = originalPlayerWidth
      playerWidth = originalPlayerHeight
      velocityX, velocityY = player.body:getLinearVelocity()
      changeCollider(player.body:getX(), player.body:getY() + (originalPlayerHeight/2), playerWidth, playerHeight)
      player.body:setLinearVelocity(velocityX, velocityY)
    end
  elseif not love.keyboard.isDown("s") and isSliding == true then
    playerHeight = originalPlayerHeight
    playerWidth = originalPlayerWidth
    isSliding = false
    velocityX, velocityY = player.body:getLinearVelocity()
    changeCollider(player.body:getX(), player.body:getY() - (originalPlayerHeight/3), playerWidth, playerHeight)
    player.body:setLinearVelocity(velocityX, velocityY)
end
end

return player
